// Annotation Tools Module
// $Author: Jeffrey J Weimer$
// SVN History: $Revision: 1344 $ on $Date: 2010-07-16 07:45:57 -0700 (Fri, 16 Jul 2010) $

#pragma rtGlobals=1
#pragma IgorVersion=6.20
#pragma version=1.0

#pragma IndependentModule=AnnotationTools

#include "ScreenSizer", optional

Static StrConstant thePackage="AnnotationTools"
Static StrConstant thePackageFolder="root:Packages:AnnotationTools"
Static StrConstant theProcedureFile = "AnnotationTools.ipf"
Static StrConstant thePackageInfo = "Control panel input to set the placement of annotations on a graph"
Static StrConstant thePackageAuthor = "Jeffrey J Weimer"
Static Constant thePackageVersion = 1.0
Static Constant hasHelp = 0
Static Constant removable = 1

// Package Constants

Static Constant pwidth = 260
Static Constant pheight = 210

// Menu

Menu "Misc"
	Submenu "Panels"
		"Annotation Tools", /Q, MakeAnnotationToolsPanel()
	end
end

// uncomment next line to edit panel + procedure file
//#define TEST

// A Function to Setup or Update

Function Setup(how)
	variable how
	
	string theCmd
	sprintf theCmd,"\"%s\",file=\"%s\",info=\"%s\",author=\"%s\",version=%f,hasHelp=%d,removable=%d,quiet=0", thePackage,theProcedureFile,thePackageInfo,thePackageAuthor,thePackageVersion,hasHelp,removable
	switch(how)
		case 0:
			sprintf theCmd, "PackageSetup(%s)", theCmd
			break
		case 1:
			sprintf theCmd, "PackageUpdate(%s)", theCmd
			break	
	endswitch
	
	Execute/P/Q/Z theCmd
	return 0
	
end

// primary functions

Function MakeAnnotationToolsPanel()

	string theCmd
	
	if (!DataFolderExists(thePackageFolder))
		DFREF cdf = GetDataFolderDFR()
		NewDataFolder/O/S root:Packages
		NewDataFolder/S $thePackageFolder
		
		variable/G posx = 0, posy = 0, dynx = 0, dyny = 0 // future expansion, dx = 0, dy = 0
		string/G currwin =""
		make/N=0/T annotelabels
		make/N=(0,2)/T annotelist
		make/N=(0,2) annotecheck, annotecheckO
		SetDataFolder cdf
		Setup(0)
	else
		Setup(1)
	endif
	
	if (WinType("AnnotationTools")==7)
		DoWindow/F AnnotationTools
		return 0
	endif

	DFREF pf = $thePackageFolder
	NVAR px = pf:posx
	NVAR py = pf:posy
	NVAR dynx = pf:dynx
	NVAR dyny = pf:dyny
//	NVAR dx = pf:dx
//	NVAR dy = pf:dy
	SVAR cw = pf:currwin
	wave/T aL = pf:annotelist
	wave/T aT = pf:annotelabels
	wave ac = pf:annotecheck
	wave acO = pf:annotecheckO

	// put up panel

	if (exists("AnnotationTools#ScreenSizer#Initialize")==6)
		sprintf theCmd "AnnotationTools#ScreenSizer#MakePanel(65,10,%d,%d,\"Annotation Tools\",other=\"/K=1/N=AnnotationTools\")", pwidth, pheight
		Execute/Q/Z theCmd
	else	
		NewPanel/W=(10,10,10+pwidth,10+pheight)/K=1/N=AnnotationTools as "Annotation Tools"
	endif
#ifndef TEST
	ModifyPanel/W=AnnotationTools noEdit=1, fixedSize=1
#endif
	
	// initialize waves and parameters
	
	redimension/N=(0,2) aL, ac, acO
	redimension/N=0 aT
	
	px = 0; py = 0; dynx = 0; dyny = 0 // future expansion; dx = 0; dy = 0
	cw = ""

	// pictures
	
	SetDrawLayer UserBack
	DrawPICT 214,55,1.25,1.25,lockicon

	// text
	
	string theV
	sprintf theV, "v %2.1f", thePackageVersion
	SetDrawEnv fname= "Arial",fsize= 8
	DrawText 5,15, theV

	// buttons

	Button refresh,pos={130,52},size={55,20},proc=ButtonProc,title="Refresh"
	Button doity,pos={25,39},size={20,20},proc=ButtonProc,fSize=16,title="Y"
	Button doitx,pos={47,11},size={20,20},proc=ButtonProc,fSize=16,title="X"
	Button showhelp, pos={190,186},size={20,20}, proc=ButtonProc, title="?"
	Button removeme, pos={220,186},size={20,20}, proc=ButtonProc, title="X"

	// sliders

	Slider setx,pos={70,3},size={150,50},proc=SliderProc,fSize=10
	Slider setx,limits={0,100,1},value= 0,side= 2,vert= 0,ticks= 10
	Slider sety,pos={3,58},size={57,120},proc=SliderProc,fSize=10
	Slider sety,limits={100,0,1},value= 0,side= 2,ticks= 10

	// value displays
	
	ValDisplay xpos,pos={220,10},size={30,14},format="%3.0f"
	ValDisplay xpos,limits={0,0,0},barmisc={0,1000}, value=_NUM:px
	ValDisplay ypos,pos={7,184},size={30,14},format="%3.0f"
	ValDisplay ypos,limits={0,0,0},barmisc={0,1000}, value=_NUM:py

	// check boxes
	
	CheckBox dynx,pos={26,14},size={21,14},value= 0,proc=CheckProc,title="!",variable=dynx
	CheckBox dyny,pos={4,42},size={21,14},value= 0,proc=CheckProc,title="!",variable=dyny
//	CheckBox cbdeltax,pos={219,27},size={25,15},title="\\F'Symbol'D",value= 0, variable=dx
//	CheckBox cbdeltaty,pos={41,184},size={25,15},title="\\F'Symbol'D",value= 0, variable=dy
	CheckBox setunset,pos={70,56},size={16,14},proc=CheckProc,title="",value=0

	// list box
	
	ListBox annotelist,pos={67,75},size={186,107},widths={140,20}, proc=ListBoxProc
	ListBox annotelist,listWave=aL, selWave=ac
	
	STRUCT WMButtonAction ba
	ba.ctrlName = "refresh"
	ba.eventCode = 2
	ButtonProc(ba)
	return 0
end

// handle the sliders on the panel

Function SliderProc(sa) : SliderControl
	STRUCT WMSliderAction &sa

	DFREF pf = $thePackageFolder
	
	NVAR xp = pf:posx
	NVAR yp = pf:posy
	NVAR dynx = pf:dynx
	NVAR dyny = pf:dyny
	
	switch( sa.eventCode )
		case -1: // kill
			break
		default:
			if( sa.eventCode & 1 ) // value set
				strswitch(sa.ctrlName)
					// slider for moving x
					case "setx":
						xp = sa.curval
						ValDisplay xpos value=_NUM:xp
						if (dynx)
							MoveAnnotations(xp,0)
						endif
						break
					// slider for moving y
					case "sety":
						yp = sa.curval
						ValDisplay ypos value=_NUM:yp
						if (dyny)
							MoveAnnotations(yp,1)
						endif
						break
				endswitch
			endif
			break
	endswitch

	return 0
End

// handle the check boxes on the panel

Function CheckProc(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba

	DFREF pf = $thePackageFolder

	wave ac = pf:annotecheck

	NVAR xp = pf:posx
	NVAR yp = pf:posy
	
	switch( cba.eventCode )
		case 2: // mouse up
			strswitch(cba.ctrlName)
				// set/unset movement checkbox depending on dynamic state
				case "dynx":
					Button doitx, disable=cba.checked*2
					if (cba.checked)
						MoveAnnotations(xp,0)
					endif
					break
				case "dyny":
					Button doity, disable=cba.checked*2
					if (cba.checked)
						MoveAnnotations(yp,1)
					endif
					break
				// set/unset all annotation checkboxes
				case "setunset":
					ac[][0] = (ac[p][1]==32) ? 2^5 + (cba.checked*16) : 0
					break
			endswitch
			break
	endswitch

	return 0
End

// handle all buttons on panel

Function ButtonProc(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	DFREF pf = $thePackageFolder
	DFREF ptf = root:Packages:PackageTools:Keys

	wave/T aL = pf:annotelabels
	wave/T aT = pf:annotelist
	wave ac = pf:annotecheck
	wave acO = pf:annotecheckO
	
	NVAR xp = pf:posx
	NVAR yp = pf:posy
	NVAR dynx = pf:dynx
	NVAR dyny = pf:dyny
	SVAR cw = pf:currwin
	
	string aList, theOne
	variable nItems, ic

	switch( ba.eventCode )
		case 2: // mouse up
			strswitch(ba.ctrlName)
				// refresh the listing
				case "refresh":
					if (strlen(WinList("*",";","WIN:1"))==0)
						dynx = 0; dyny = 0; xp = 0; yp = 0; cw = ""
						redimension/N=(0) aL
						redimension/N=(0,2) aT, ac, acO
						Slider setx value = 0
						Slider sety value = 0
						CheckBox setunset, value=0
						Button doitx disable = 0
						Button doity disable =0
						ValDisplay xpos value=_NUM:xp
						ValDisplay ypos value=_NUM:yp
						break
					endif
					cw = WinName(0,1,1)
					aList = AnnotationList("")
					nItems = ItemsInList(aList)
					redimension/N=(nItems) aL
					redimension/N=(nItems,2) aT, ac, acO
					ac = 2^5
					acO = ac
					for (ic=0;ic<nItems;ic+=1)
						theOne = StringFromList(ic,aList)
						aL[ic] = theOne
						aT[ic][0] = theOne + " : " + GetText(theOne)
						TextBox/C/N=$theOne/A=LT/Z=0
					endfor
					CheckBox setunset, value=0
					break
				// move annotation(s)
				case "doitx":
					MoveAnnotations(xp,0)
					break
				case "doity":
					MoveAnnotations(yp,1)
					break
				case "showhelp":
					if (DataFolderRefStatus(ptf)!=0)
						SVAR/Z/SDFR=root:Packages:PackageTools:Keys AnnotationTools
						if (!SVAR_exists(AnnotationTools))
							Setup(0)
						endif
					else
						DoAlert 0, "No help is currently available."
						break
					endif
					sprintf theOne, "PackageHelp(\"%s\")", thePackage
					Execute/P/Q/Z theOne
					break
				case "removeme":
					if (DataFolderRefStatus(ptf)!=0)
						SVAR/Z/SDFR=root:Packages:PackageTools:Keys AnnotationTools
						if (!SVAR_exists(AnnotationTools))
							Setup(0)
						endif
					else
						DoAlert 0, "You MUST install PackageTools to remove this package."
						break
					endif
					sprintf theOne, "PackageRemove(\"%s\")", thePackage
					Execute/P/Q/Z theOne
					theOne = "AnnotationTools#PossiblyRemovePanel()"
					Execute/P/Q/Z theOne
					break	
			endswitch
			break
	endswitch

	return 0
End

// handle the list box

Function ListBoxProc(lba) : ListBoxControl
	STRUCT WMListboxAction &lba

	DFREF pf = $thePackageFolder

	wave/T aL = pf:annotelabels
	wave ac = pf:annotecheck
	wave acO = pf:annotecheckO
			
	NVAR xp = pf:posx
	NVAR yp = pf:posy
	NVAR dynx = pf:dynx
	NVAR dyny = pf:dyny

//	Variable row = lba.row
//	Variable col = lba.col
//	WAVE/T/Z listWave = lba.listWave
//	WAVE/Z selWave = lba.selWave

	variable ic, nItems
	string theOne
	
	switch( lba.eventCode )
		case -1: // control being killed
			break
		case 2: // mouse up
			nItems = numpnts(aL)
			for (ic=0;ic<nItems;ic+=1)
				// lock/unlock
				if (ac[ic][1] != acO[ic][1])
					theOne = aL[ic]
					if (ac[ic][1] > 32)
						ac[ic][0] = 0
						TextBox/C/N=$theOne/Z=1
					else
						ac[ic][0] = 32
						TextBox/C/N=$theOne/Z=0
					endif
				else
					// move x or y
					if ((ac[ic][0]/32) > 1)
						theOne = aL[ic]
						if (dynx)
							TextBox/C/N=$theOne/A=LT/X=(xp)
						endif
						if (dyny)
							TextBox/C/N=$theOne/A=LT/Y=(yp)		
						endif
					endif
				endif
			endfor
			acO = ac
			break
	endswitch

	return 0
End

// move annotation(s) by a given value in x or y
// annotation labels to be moved are stored in the aL wave
// whether to move them is defined by the ac wave

Function MoveAnnotations(value,xory)
	variable value, xory

	wave/T aL = root:Packages:AnnotationTools:annotelabels
	wave ac = root:Packages:AnnotationTools:annotecheck
	
	string theOne
	variable ic, nItems
	
	nItems = DimSize(ac,0)
	
	for (ic=0;ic<nItems;ic+=1)
		if ((ac[ic][0]/32)>1)
			theOne = aL[ic]
			if (xory==0)
				TextBox/C/N=$theOne/A=LT/X=(value)
			else
				TextBox/C/N=$theOne/A=LT/Y=(value)		
			endif
		endif
	endfor
	
	return 0
end

// get the label text for a given label name

Function/S GetText(anname)
	string anname
	
	string atext = StringByKey("Text",AnnotationInfo("",anname,1))
	return (ReplaceString("\r",atext[0,25],"-"))
	
end

// possibly remove panel if the package folder contents disappear

Function PossiblyRemovePanel()

	DFREF pf = $thePackageFolder

	SVAR cw = pf:currwin
	if (!SVAR_exists(cw))
		KillWindow AnnotationTools
	endif
	return 0
end

// lock icon for panel
// PNG: width= 16, height= 16
Picture lockicon
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!!1!!!!1#R18/!$Ch4@/p9/1J'u#<-Xq0O\\MI;&%$bHf($EP/s#XE/Y
	W*L*-a<@r5K(`X%8HBjIc?[YS07/X22P/W(ZS4/!U'#cu\n%H[A_dO*0&Bo\TA1Y03abmWu%pTtd^8
	On@nX*[%uq0h/Z!<62rSClQL\uWC;p_j*<J%\`$;R<\_A$JOdBJc1R/$Z.U]@!(=r]6O;@r$CFhgbF
	BPOGAn-o&$[hL=G9qYs$!e(3+4^XU[$KGhDB`^WH>%M%?_(k;2Y^p`On^qjDqm0P/4mn5!kho`P)p%
	Mfgc$ea2aM#Vg9hgnRH8lnIkj=?<]RDZY=*o7rQd3\#VP^5fU#<L!:(5s<`FAXC4cr:6GW^H01^s/,
	H]EbG+]0#jbhJC\E#`M1ICaNH0rp]cUtfCIVVDR[V%f;B_g4X?BHiQ$$.C`_o[QImcGaip)\.!/dL3
	]^NXQ2aqZQ9fg'.7!9N1cueCC*rR%Hr7J!%9^^2s&r!;o<?p</bMQ,s$I7eL3%"Zcn?bsRSPXlahXC
	bWiepM$65#b25+K=<DTijC=D(/W0SfmpL3b2HNR4dbg(4F&"k&'tAO#!`fO`ZGLP97V$_SL[#TLRf/
	/^aK2?^tJcZJ4Y>O"aF1#Ee(U7j<C-H5j2d`=*/!dJ<h\lYR]SC!U:%prr@iTOuV2U%b:]_<WH=)JB
	@j]Vm\]l866,K<<*i+Qj',dVWJ&tU'P--.oSH:=\h!5qHrM(*bZc>bZT1"z8OZBBY!QNJ
	ASCII85End
End
